@@ -1,7 +1,7 @@
 #' Database of information for incubation of turtles
 #' @title Version of database of TSD information for reptiles
 #' @author Marc Girondot \email{marc.girondot@@universite-paris-saclay.fr}
-#' @return The date of the lastest updated version
+#' @return The date of the latest updated version
 #' @description Return the date of the most recent update of the ROSIE database.
 #' @family Functions for temperature-dependent sex determination
 #' @examples
@@ -13,5 +13,6 @@
 
 ROSIE.version <- function() {
   # data("DatabaseTSD", package = "embryogrowth")
-  return("1.0.3")
+  cat("This is a modified version of the ROSIE database.")
+  return("1.0.3 modified 2025-06-03")
 }
